@@ -107,6 +107,9 @@ class Config:
         self.device_name: str = self._get_cfg(
             ["matrix", "device_name"], default="matrix-alertbot"
         )
+        self.user_token_file: str = self._get_cfg(
+            ["matrix", "user_token_file"], default="token.json"
+        )
         self.homeserver_url: str = self._get_cfg(["matrix", "url"], required=True)
         self.allowed_rooms: list = self._get_cfg(
             ["matrix", "allowed_rooms"], required=True
