@@ -1,7 +1,11 @@
 # This file holds custom error types that you can define for your application.
 
 
-class ConfigError(Exception):
+class MatrixAlertbotError(Exception):
+    pass
+
+
+class ConfigError(MatrixAlertbotError):
     """An error encountered during reading the config file."""
 
     pass
@@ -25,7 +29,7 @@ class RequiredConfigKeyError(ConfigError):
     pass
 
 
-class AlertmanagerError(Exception):
+class AlertmanagerError(MatrixAlertbotError):
     """An error encountered with Alertmanager."""
 
     pass
