@@ -85,7 +85,7 @@ class CommandTestCase(unittest.IsolatedAsyncioTestCase):
         self.fake_matrix_client = Mock(spec=nio.AsyncClient)
         self.fake_matrix_client.user = "@fake_user:example.com"
         # Pretend that attempting to send a message is always successful
-        self.fake_matrix_client.room_send.return_value = make_awaitable(None)
+        self.fake_matrix_client.room_send.return_value = make_awaitable()
 
         self.fake_cache = MagicMock(spec=Cache)
         self.fake_cache.__getitem__.side_effect = cache_get_item
