@@ -11,7 +11,7 @@ def run_coroutine(result: Awaitable[Any]) -> Any:
     return result
 
 
-def make_awaitable(result: Any) -> Awaitable[Any]:
+def make_awaitable(result: Any = None) -> Awaitable[Any]:
     """
     Makes an awaitable, suitable for mocking an `async` function.
     This uses Futures as they can be awaited multiple times so can be returned
